#!/bin/bash
#set -n		# read but do not execute commands (check on syntax errors)
#
# File: /usr/local/share/ksm-gpgswitch/checkproc.sh
#
# Checks for (shell) processes by pathname.
#
#    Copyright (C) 2017  Juergen Kaesmann (JK)
#
##############################################################################
# GPL NOTICE
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
# EXIT CODES
# Standard

##############################################################################
# SOURCE FILES

#. /usr/local/etc/main.conf				# config main
#. $LLIBDIR/libfuncstd-sh				# functions standard
. /usr/local/share/ksm-gpgswitch/libfuncstd-sh
#. /usr/local/etc/sample.conf				# config sample
#. ~/.samplerc						# config user

##############################################################################
# DECLARATION OF CONSTANTS AND VARIABLES

#------------------STANDARD CONSTANTS AND VARIABLES---------------------------
readonly package=GPGswitch				# package name
readonly vinfopath='/usr/local/share/ksm-gpgswitch'	# path to package dir
readonly version=$(/bin/cat $vinfopath/VERSION)		# package version
readonly release=$(/bin/cat $vinfopath/RELEASE)		# package release
readonly scriptname=$(basename $0)			# name of script
#readonly tempfile=$TEMPDIR/$scriptname.$UID.tmp	# temporary file name
#tempfile=$(mktemp $TEMPDIR/$scriptname.$UID.XXXXXX)	# temporary file name
readonly debuglog="/bin/logger -p user.debug -t $scriptname -- DEBUG:"	# log dbg msg
readonly errlog="/bin/logger -p user.err -t $scriptname -- ERROR:"	# log err msg
readonly infolog="/bin/logger -p user.info -t $scriptname -- INFO:"	# log inf msg
cmd=''							# sub command
debug=no						# debug=yes/no
verbose=''						# verbose=yes/''/no

#------------------USER FILL IN AFTER INSTALLATION----------------------------
#echo "$scriptname line 59: Setup following 2 vars. Then comment out this line." >&2; exit 1
#readonly var1='Dummy1'
#var2='Dummy2'

#-----------------------USER-CONSTANTS----------------------------------------
# STRINGS
#readonly const=string

# NUMBERS
#readonly const=value

#-----------------------USER-VARIABLES----------------------------------------
# STRINGS
#args=''				# all args from cmdline
arg1=''					# 1. arg from cmdline
#opt_i=''				# option -i (arg)
#opt_o=''				# option -o (yes)
#infile=''				# option --infile (arg)
#outfile=''				# option --outfile (arg)
#sample=''				# option --sample (yes/no/arg)
opt_p=''				# option -p (arg)

# NUMBERS
err=0					# error code

# ARRAYS
# Indexed arrays are always zero based string arrays.
# If index starts with 1 and all members are assigned then
# count of members of array = last index of array
#declare -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array

##############################################################################
# FUNCTIONS
##############################################################################

#=============================================================================
function helptext ()
{
# Output helptext
# Uses cat, echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG helptext: $# $@" >&2
	$debuglog helptext: $# $@
fi

cat <<EOT
NAME
$scriptname - Checks for (shell) processes by pathname.

SYNOPSIS
$scriptname [options] [process]

DESCRIPTION
Checks for running (shell) processes that use the specified executable.

$scriptname does not use the PID to verify a process but the full
path of the corresponding program which is used to identify the
executable. To do so the PID read from the pidfile is used to search
the process table for processes with an executable that matches the
specified pathname.

Defaults:  Check processes.

OPTIONS
Gnu style options:

  -h|--help    = Output help, then exit.
  -V|--version = Output version info, then exit.
  -d|--debug   = Turn on DEBUG mode.
  -q|--quiet   = Be quiet.
  -v|--verbose = Be verbose.
  -p <pidfile> = PID file to use. The pid read from this file is
                 compared against the pids of the processes that uses
                 the specified binary. If the pid_file does not exist,
                 $scriptname assumes that the daemon is not running.
  --           = End of options.

ARGUMENTS
Give if needed:

  process = Pathname of process to check

EXAMPLES
For instance:

  Check process /usr/local/bin/myprog. Use default PID-File:
  $scriptname /usr/local/bin/myprog

  Check process /usr/local/bin/myprog. Use PID-File /var/myrun/myprog:
  $scriptname -p /var/myrun/myprog /usr/local/bin/myprog

EXIT STATUS
On program exit:

  Return values acc. to LSB for status:
    0     - Program is running
    1     - No process but pid file found
    3     - No process and no pid file found
    4     - Program is not installed
  150     - Other error
  5--199  - reserved (5--99 LSB, 100--149 distrib, 150--199 appl)

FILES
Used directories and files:

  Default Pid file = /var/run/<basename arg1>.pid

SEE ALSO
Related manpages:

  showerrmsg(1).

AUTHORS
Juergen Kaesmann <juergen@kaesmann-online.de>
EOT
#  -o|--output  = Output goes to STDOUT.
#  -i f         = Input from file <f>.
#  --infile=f   = Input from file <f>.

return
}

#=============================================================================
function mainproc ()
{
# Sample function.
# par1 = 
# Uses echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG main proc: $# $@" >&2
	$debuglog main proc: $# $@
fi

local b='' pn='' pidfile=''
local pid=0 pid2=0 ret=0
#local -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array
#s=$(date "+%d.%m.%Y %H:%M:%S")		# DD.MM.YYYY  HH:MM:SS

test "$verbose" == yes && echo -e "\ncheckproc mainproc: Argument=$arg1"
if [ -e "$arg1" ]; then
	# setup pidfile
	b=$(basename $arg1)
	if [ -z "$opt_p" ]; then
		pidfile=/var/run/$b.pid
	else
		pidfile="$opt_p"
	fi
	test "$verbose" == yes && echo -e "checkproc mainproc: Using pidfile=$pidfile"

	# do it
	test -e "$pidfile" && pid2=$(/bin/cat $pidfile)	# get pid from pidfile
	ret=$?
	if [ "$ret" -eq 0 ]; then			# got pid2
		test "$verbose" == yes && echo "checkproc mainproc: pid2 $pid2 from pidfile"
		Get_Path_PID $pid2 pn			# get path from pid2
		ret=$?
		if [ "$ret" -eq 0 ]; then		# got path
			test "$verbose" == yes && echo "checkproc mainproc: path $pn from pid2 $pid2 with ps"
			Get_PID $pn pid			# get pid from path
			ret=$?
			if [ "$ret" -eq 0 ]; then	# got pid
				test "$verbose" == yes && echo "checkproc mainproc: pid $pid with ps from $pn"
				if [ "$pid" == "$pid2" ]; then # pids are equal
					test "$verbose" == yes && echo "checkproc mainproc: pid $pid equal pid2 $pid2 - proc is running"
					ret=0
				else			# pids diff, same path
					test "$verbose" == yes && echo "checkproc mainproc: pid $pid unequal pid2 $pid2 - Error"
					ret=150
				fi
			else				# no pid found
				test "$verbose" == yes && echo "checkproc mainproc: no pid from $pn"
				ret=1
			fi
		else					# no path found
			test "$verbose" == yes && echo "checkproc mainproc: no path from pid2 $pid2"
			ret=1
		fi
	else						# no pidfile
		test "$verbose" == yes && echo "checkproc mainproc: no pidfile $pidfile"
		ret=1
	fi
else
	test "$verbose" == yes && echo "checkproc mainproc: program $arg1 not installed"
	ret=4
fi

return $ret
}

##############################################################################
# GET OPTIONS
##############################################################################
# Give 2 strings and all cmdline_args to func Get Opt
# (POSIX_options Gnu_long_options cmdline_args):
# Get Opt 'hVdqvi:m:' 'help version debug quiet verbose infile= myval=' $@
#
# A ':' after a POSIX option means this option needs an arg.
# A '=' after a Gnu long option means this option needs an arg.
#
# All cmd/options have to be initialised to ''!
#
# POSIX options are classified in 2 categories:
#   a) cmds (mutually exclusive). All cmds are given to var 'cmd'.
#   b) options (as many as you like). Each option is given to it's own var.
# As a general rule: If cmds/options are given multiple then last given wins.
#
# Options can take 1 argument (no whitespace in arg). If you want to give more
# than 1 arg or want to use multiple strings as 1 arg then concatenate them
# together with a delimiter like ',' or ';' or '.'
# Use a delimiter which will not be part of the arg:
# sampleprog -m val1,val2
# sampleprog --myval=val1,val2
# It's at the programmers responsibility to do meaningfull things with the
# content of the options.
#
# If err != 0 then all args are processed, else continue to look for options.
# When Get Opt finishes then all options are removed from cmdline.
#
if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG Parse Commandline: $# $@" >&2
	$debuglog Parse Commandline: $# $@
fi
#
err=0
while [ "$err" -eq 0 ]; do
	GetOpt 'hVdqvp:' 'help version debug quiet verbose' $@
	err=$?
	if [ $err -eq 0 ]; then
		case "$_OPTNAME_" in
		    h|help)	cmd="-h";;
		    V|version)	cmd="-V";;
#		    a)		cmd="-${_OPTNAME_}";;
		    d|debug)	debug=yes;;
		    q|quiet)	verbose=no;;
		    v|verbose)	verbose=yes;;
#		    o|output)	opt_o=yes;;
#		    i|infile)	opt_i="$_OPTVAL_";;
		    p)		opt_p="$_OPTVAL_";;
		    --)		break;;
		    :)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[8]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[8]} -${_OPTVAL_}
			fi; exit 1;;
		    ?)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[4]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[4]} -${_OPTVAL_}
			fi; exit 1;;
		    *)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}
			fi; exit 1;;
		esac
	fi
done
err=0
#if [ -z "$cmd" ]; then					# error no cmd given
#	if [ "$verbose" != no ]; then
#		echo "${ErrMsg[7]}" >&2
#	else
#		$errlog ${ErrMsg[7]}
#	fi
#	exit 7
#fi
shift ${_OPTIDX_}				# rm all opts from cmdline

##############################################################################
# CHECK LINK NAMES

##############################################################################
# CHECK OPTIONS

##############################################################################
# CHECK VERSION

##############################################################################
# CHECK ARGUMENTS
#
case "$cmd" in
    '')
	if [ "$#" -eq 1 ]; then				# parameter count ok
		arg1="$1"
		shift
		#args="$@"
		#shift $#
	else						# error in param count
		if [ "$verbose" != no ]; then
			echo "${ErrMsg[2]}" >&2
		else
			$errlog ${ErrMsg[2]}
		fi
		exit 2
	fi
	;;
    *)
	if [ "$#" -ne 0 ]; then				# error in param count
		if [ "$verbose" != no ]; then
			echo "${ErrMsg[2]}" >&2
		else
			$errlog ${ErrMsg[2]}
		fi
		exit 2
	fi
	;;
esac

##############################################################################
# MAINPROGRAM
##############################################################################
#
if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG Main: $# $@" >&2
	$debuglog Main: $# $@
fi

# Check what to do (sub commands, options and args are removed from cmdline)
case "$cmd" in
    '')							# no cmd given
	mainproc
	err=$?
	;;
    -h)							# cmd -h
	helptext
	err=0
	;;
    -V)							# cmd -V
	echo "$scriptname (${package}) ${version}-${release}"
	err=0
	;;
    *)							# other cmds = error
	if [ "$verbose" != no ]; then
		echo "${ErrMsg[6]}" >&2
	else
		$errlog ${ErrMsg[6]}
	fi
	err=1
	;;
esac

exit $err
